function [K, Q_u, eigenValues] = getStiffnessMatrix_3N(block, qt)
%%  Parameters
    [ numberOfNodes , ~ ] = size(block.nodes);
    numberOfNodes = numberOfNodes - 1;
    [ numberOfElements , eleCol ] = size(block.elements);
    numberOfDOF = numberOfNodes * qt.DOFPerNode;
    qt.nodesPerElement = eleCol / 2;
    
    poissonRatio = 0.25;
    constitutiveLaw = 2; %1 = Plain strain 2 = plain stress 
    
%%  Initialize variables
    %Coefficient matrices
    E0_block = zeros(numberOfDOF, numberOfDOF);
    E1_block = zeros(numberOfDOF, numberOfDOF);
    E2_block = zeros(numberOfDOF, numberOfDOF);
    
%%  Numerical integration values
    [ samplingPoints , weights ] = integrationParameters1D ( qt.nodesPerElement , qt.quadratureType);
    numberOfSamplingPoints = length(samplingPoints);
    
%%  Coefficient matrices assembly loop 
    %loop over all elements --> fully assembled EX_subdomain
    for i = 1:1:numberOfElements

        E0_temp = zeros( qt.DOFPerNode * qt.nodesPerElement, qt.DOFPerNode * qt.nodesPerElement );
        E1_temp = zeros( qt.DOFPerNode * qt.nodesPerElement, qt.DOFPerNode * qt.nodesPerElement );
        E2_temp = zeros( qt.DOFPerNode * qt.nodesPerElement, qt.DOFPerNode * qt.nodesPerElement );

        %recall (x,y)-coordinates of each elements nodes
        xCoord = [block.elements(i,1);block.elements(i,3);block.elements(i,5)] - block.sCX;
        yCoord = [block.elements(i,2);block.elements(i,4);block.elements(i,6)] - block.sCY;

        %recall material properties for each element to form corresponding matrix
        [ D ] = constitutiveMatrix2D( block.EMod , poissonRatio , constitutiveLaw );

        %loop over the integration points --> fully integrated coefficient matrices EX_temp
        for j = 1:1:numberOfSamplingPoints
            %recall specific shape functions and their derivatives
            samplingPoint = samplingPoints(j);
            [shapeFunction , shapeFunctionDerivative] = shapeFunction1D (qt.nodesPerElement , samplingPoint);

            numberOfShapeFunctions = length(shapeFunction);

            %recall specific integration weights
            weight = weights(j);

            %recall interpolation of cartesian coordinates (x,y) and their derivates
            x   = shapeFunction * xCoord;
            y   = shapeFunction * yCoord;
            x_n = shapeFunctionDerivative * xCoord;
            y_n = shapeFunctionDerivative * yCoord;

            %calculate specific jacobian and it's determinant
            detJ = x * y_n - y * x_n;
            %correct for negative determinant --> don't know why it is negative all of a sudden???
            detJ = -detJ;
            %specify shape function for each DOF
                %initialize shape function and it's derivatives in 2D
                    shapeFunction2D           = zeros( qt.DOFPerNode , qt.DOFPerNode * qt.nodesPerElement );
                    shapeFunctionDerivative2D = zeros( qt.DOFPerNode , qt.DOFPerNode * qt.nodesPerElement );

                %construct shape functions and associated derivaties in 2D
                    positionCounter = 1;
                    for k = 1:1:numberOfShapeFunctions
                        %abbreviation for simple notation
                        positionRow = 1:qt.DOFPerNode;
                        positionCol = positionCounter : positionCounter + qt.DOFPerNode - 1;

                        shapeFunction2D(positionRow,positionCol)           = shapeFunction(k)           * eye(2);
                        shapeFunctionDerivative2D(positionRow,positionCol) = shapeFunctionDerivative(k) * eye(2);
                        positionCounter = positionCounter + qt.DOFPerNode;
                    end

            %calculate strain desplacement matrices B1 B2
            B1 =   1/detJ * [y_n ,   0 ;...
                               0 , -x_n;...
                            -x_n ,  y_n] * shapeFunction2D;
            B2 =   1/detJ * [ -y ,   0 ;...
                               0 ,   x ;...
                               x ,  -y ] * shapeFunctionDerivative2D;

            %find specific contributrion to coefficient matrices
            E0_temp = E0_temp + transpose(B1) * D * B1 * detJ * weight;
            E1_temp = E1_temp + transpose(B2) * D * B1 * detJ * weight;
            E2_temp = E2_temp + transpose(B2) * D * B2 * detJ * weight;

        end

        %save coefficient matrix contribution in correct position
            %determine saving position
                ind = block.lConn(i,:);

            %implement proper storing procedure
                E0_block(ind, ind) = E0_block(ind, ind) + E0_temp(:,:);
                E1_block(ind, ind) = E1_block(ind, ind) + E1_temp(:,:);
                E2_block(ind, ind) = E2_block(ind, ind) + E2_temp(:,:);
    end

%%  Construct Hamiltonian
    %form Hamiltonian matrix Z
    invE_0 = E0_block\eye(size(E0_block));

    Z11 = invE_0 * E1_block';
    Z12 = - invE_0;
    Z21 = - E2_block + (E1_block * invE_0 * E1_block');
    Z22 = - E1_block * invE_0;

    Z = [Z11, Z12; Z21, Z22];
    
    %extract parameters
        [rowZ, ~ ] = size(Z);
        
%%  Perform eigenvalue decomposition
    [eigenVector , eigenValues] = eig(Z);
    
    %correct the zero block
        %eigenValues
            %extract eigenvalues and find their sorting
            eigenValues = diag(eigenValues);
            [~,sortingIndex]=sort(real(eigenValues),'ascend');
            eigenValues = real(eigenValues(sortingIndex));
            eigenValues = eigenValues(1:end/2);
            eigenValues(end-1:end) = [0,0];
            
        %eigenVectors
            %sort eigenvectors to correspond with eigenvalues
            eigenVector = eigenVector(:,sortingIndex);
            %adjust translational modes
                eigV1 = zeros(rowZ,1);
                eigV1(1:2:end) = ones(rowZ/2,1);
                eigV1 = eigV1/norm(eigV1);
                eigV2 = zeros(rowZ,1);
                eigV2(2:2:end) = ones(rowZ/2,1);
                eigV2 = eigV2/norm(eigV2);
                lEig = length(eigenValues);
                eigenVector(:,lEig-1) = 1*eigV1+2*eigV2;
                eigenVector(:,lEig  ) = 3*eigV1+2*eigV2;
                eigenVector(lEig+1:end,lEig-1:lEig) = zeros;
                eigenVector(:,lEig-1) = eigenVector(:,lEig-1)/norm(eigenVector(:,lEig-1));
                eigenVector(:,lEig)   = eigenVector(:,lEig  )/norm(eigenVector(:,lEig  )); 

%%  Form Stiffness matrix
    %extract relevant positions
    Q_u = eigenVector(            1:  numberOfDOF, 1:numberOfDOF);
    Q_q = eigenVector(numberOfDOF+1:2*numberOfDOF, 1:numberOfDOF);
    
    %evaluate stiffness matrix
    K = real( Q_q / Q_u );
    K = (1/2)*(K + K');
end